import { Connection, PublicKey, Transaction } from "@solana/web3.js";
import { BN } from "@coral-xyz/anchor";
import { Context, FlexibleStakingInfo, PDA } from "..";
export declare class StakingClient {
    ctx: Context;
    pda: PDA;
    constructor(ctx: Context, pda: PDA);
    depositFund(amount: BN): Promise<Transaction>;
    updateAdmin(newAdmin: PublicKey): Promise<Transaction>;
    withdrawFund(amount: BN): Promise<Transaction>;
    updateLockupVault(index: number, apy: number, lockupDuration: BN, minStakeAmount: BN): Promise<Transaction>;
    updateFlexibleVault(rewardRate: BN, withdrawalDelayTime: BN): Promise<Transaction>;
    stake(amount: BN): Promise<Transaction>;
    withdraw(amount: BN): Promise<Transaction>;
    withdrawAfterDelay(index: number): Promise<Transaction>;
    claim(): Promise<Transaction>;
    lockup(amount: BN, vaultIndex: number): Promise<Transaction>;
    unlock(index: number, vaultIndex: number): Promise<Transaction>;
    getUserFlexibleStakingInfo(filter?: {
        withdrew: boolean;
    }): Promise<FlexibleStakingInfo>;
    getUserLockupStakingInfo(filter?: {
        unlocked: boolean;
    }): Promise<{
        totalLockedAmount: string;
        maturedAmount: string;
        lockupStakingAccounts: ({
            bump: any;
            index: any;
            lockupVault: any;
            lockupVaultIndex: any;
            committedApy: any;
            lockedAmount: any;
            lockedAt: any;
            unlocked: any;
        } | null)[];
    }>;
}
export declare function getProtocolInfo(connection: Connection): Promise<{
    lockupApy: number[];
    lockupDuration: string[];
    minLockupAmount: string[];
    totalLockupStaked: string[];
    totalLockupReward: string[];
    admin: string;
    mint: string;
    flexibleApr: number;
    flexibleWithdrawalDelayTime: string;
    totalFlexibleStaked: string;
    totalFlexibleReward: string;
}>;
