"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StakingClient = void 0;
exports.getProtocolInfo = getProtocolInfo;
const web3_js_1 = require("@solana/web3.js");
const anchor_1 = require("@coral-xyz/anchor");
const __1 = require("..");
const fetcher_1 = require("../fetcher");
const spl_token_1 = require("@solana/spl-token");
class StakingClient {
    constructor(ctx, pda) {
        this.ctx = ctx;
        this.pda = pda;
    }
    depositFund(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get config data
            const config = this.pda.config();
            const configInfo = yield this.ctx.connection.getAccountInfo(config.key);
            if (!configInfo) {
                throw new Error("Config account not found");
            }
            const configData = yield this.ctx.program.account.config.fetch(config.key);
            const tx = new web3_js_1.Transaction();
            const senderATA = (0, spl_token_1.getAssociatedTokenAddressSync)(configData.mint, this.ctx.wallet.publicKey);
            const senderATAInfo = yield this.ctx.connection.getAccountInfo(senderATA);
            if (!senderATAInfo) {
                throw new Error("Sender associated token account not found");
            }
            const senderATAData = yield (0, spl_token_1.getAccount)(this.ctx.connection, senderATA);
            if ((new anchor_1.BN(senderATAData.amount.toString())).lt(amount)) {
                throw new Error("Insufficient token balance to deposit");
            }
            const configATA = (0, spl_token_1.getAssociatedTokenAddressSync)(configData.mint, config.key, true);
            const configATAInfo = yield this.ctx.connection.getAccountInfo(configATA);
            if (!configATAInfo) {
                tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(this.ctx.wallet.publicKey, configATA, config.key, configData.mint));
            }
            tx.add((0, spl_token_1.createTransferCheckedInstruction)(senderATA, configData.mint, configATA, this.ctx.wallet.publicKey, BigInt(amount.toString()), __1.SOLCAT_DECIMALS));
            return tx;
        });
    }
    // Admin functions
    updateAdmin(newAdmin) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.pda.config();
            const configData = yield this.ctx.fetcher.getConfig(config.key, true);
            if (!configData) {
                throw new Error("Config account not found");
            }
            if (this.ctx.wallet.publicKey.toBase58() !== configData.admin.toBase58()) {
                throw new Error("Unauthorized");
            }
            const tx = new web3_js_1.Transaction();
            const updateAdminIx = yield this.ctx.methods.updateAdmin({
                inputs: {
                    newAdmin,
                },
            });
            if (!updateAdminIx.ix) {
                throw new Error("Failed to create updating admin instruction");
            }
            tx.add(updateAdminIx.ix);
            return tx;
        });
    }
    withdrawFund(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.pda.config();
            const configInfo = yield this.ctx.connection.getAccountInfo(config.key);
            if (!configInfo) {
                throw new Error("Config account not found");
            }
            const configData = yield this.ctx.program.account.config.fetch(config.key);
            if (this.ctx.wallet.publicKey.toBase58() !== configData.admin.toBase58()) {
                throw new Error("Unauthorized");
            }
            const tx = new web3_js_1.Transaction();
            const adminATA = (0, spl_token_1.getAssociatedTokenAddressSync)(configData.mint, this.ctx.wallet.publicKey);
            const adminATAInfo = yield this.ctx.connection.getAccountInfo(adminATA);
            if (!adminATAInfo) {
                tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(this.ctx.wallet.publicKey, adminATA, this.ctx.wallet.publicKey, configData.mint));
            }
            const configATA = (0, spl_token_1.getAssociatedTokenAddressSync)(configData.mint, config.key, true);
            const configATAInfo = yield this.ctx.connection.getAccountInfo(configATA);
            if (!configATAInfo) {
                throw new Error("Config associated token account not found");
            }
            const configATAData = yield (0, spl_token_1.getAccount)(this.ctx.connection, configATA);
            if ((new anchor_1.BN(configATAData.amount.toString())).lt(amount)) {
                throw new Error("Insufficient token balance to withdraw");
            }
            const withdrawFundIx = yield this.ctx.methods.withdrawFund({
                inputs: {
                    amount,
                },
            });
            if (!withdrawFundIx.ix) {
                throw new Error("Failed to create withdrawing fund instruction");
            }
            tx.add(withdrawFundIx.ix);
            return tx;
        });
    }
    updateLockupVault(index, apy, lockupDuration, minStakeAmount) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.pda.config();
            const configInfo = yield this.ctx.connection.getAccountInfo(config.key);
            if (!configInfo) {
                throw new Error("Config account not found");
            }
            const configData = yield this.ctx.program.account.config.fetch(config.key);
            if (this.ctx.wallet.publicKey.toBase58() !== configData.admin.toBase58()) {
                throw new Error("Unauthorized");
            }
            const lockupVault = this.pda.lockupVault(new anchor_1.BN(index));
            const lockupVaultInfo = yield this.ctx.connection.getAccountInfo(lockupVault.key);
            if (!lockupVaultInfo) {
                throw new Error("Lockup vault is not initialized");
            }
            const tx = new web3_js_1.Transaction();
            const updateLockupVaultIx = yield this.ctx.methods.updateLockupVault({
                inputs: {
                    index,
                    apy,
                    lockupDuration,
                    minStakeAmount,
                },
            });
            if (!updateLockupVaultIx.ix) {
                throw new Error("Failed to create updating lockup vault instruction");
            }
            tx.add(updateLockupVaultIx.ix);
            return tx;
        });
    }
    updateFlexibleVault(rewardRate, withdrawalDelayTime) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.pda.config();
            const configInfo = yield this.ctx.connection.getAccountInfo(config.key);
            if (!configInfo) {
                throw new Error("Config account not found");
            }
            const configData = yield this.ctx.program.account.config.fetch(config.key);
            if (this.ctx.wallet.publicKey.toBase58() !== configData.admin.toBase58()) {
                throw new Error("Unauthorized");
            }
            const flexibleVault = this.pda.flexibleVault();
            const flexibleVaultInfo = yield this.ctx.connection.getAccountInfo(flexibleVault.key);
            if (!flexibleVaultInfo) {
                throw new Error("Flexible vault is not initialized");
            }
            const tx = new web3_js_1.Transaction();
            const updateFlexibleVaultIx = yield this.ctx.methods.updateFlexibleVault({
                inputs: {
                    rewardRate,
                    withdrawalDelayTime,
                },
            });
            if (!updateFlexibleVaultIx.ix) {
                throw new Error("Failed to create updating flexible vault instruction");
            }
            tx.add(updateFlexibleVaultIx.ix);
            return tx;
        });
    }
    // User functions
    stake(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const flexibleAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleAccount.key);
            if (!flexibleAccountInfo) {
                const initFlexibleAccountIx = yield this.ctx.methods.initFlexibleAccount({
                    accounts: {
                        user: this.ctx.wallet.publicKey,
                    },
                    inputs: {
                        bump: flexibleAccount.bump,
                    },
                });
                if (!initFlexibleAccountIx.ix) {
                    throw new Error("Failed to create initializing flexible account instruction");
                }
                tx.add(initFlexibleAccountIx.ix);
            }
            const stakeIx = yield this.ctx.methods.stake({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    amount,
                },
            });
            if (!stakeIx.ix) {
                throw new Error("Failed to create staking instruction");
            }
            tx.add(stakeIx.ix);
            return tx;
        });
    }
    withdraw(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const flexibleAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleAccount.key);
            if (!flexibleAccountInfo) {
                throw new Error("Ensure user has staked before withdrawing");
            }
            const userAccount = this.pda.user(this.ctx.wallet.publicKey);
            const userAccountInfo = yield this.ctx.connection.getAccountInfo(userAccount.key);
            let withdrawIndex = new anchor_1.BN(0);
            if (!userAccountInfo) {
                const initUserIx = yield this.ctx.methods.initUser({
                    accounts: {
                        user: this.ctx.wallet.publicKey,
                    },
                    inputs: {
                        bump: userAccount.bump,
                    },
                });
                if (!initUserIx.ix) {
                    throw new Error("Failed to create initializing user instruction");
                }
                tx.add(initUserIx.ix);
            }
            else {
                const userData = yield this.ctx.program.account.user.fetch(userAccount.key);
                withdrawIndex = new anchor_1.BN(userData.latestWithdrawIndex);
            }
            const withdrawPosition = this.pda.withdrawPosition(this.ctx.wallet.publicKey, new anchor_1.BN(withdrawIndex));
            const withdrawPositionInfo = yield this.ctx.connection.getAccountInfo(withdrawPosition.key);
            if (withdrawPositionInfo) {
                throw new Error("Withdraw position index is already taken");
            }
            const withdrawIx = yield this.ctx.methods.withdraw({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    withdrawPositionBump: withdrawPosition.bump,
                    index: withdrawIndex.toNumber(),
                    amount,
                },
            });
            if (!withdrawIx.ix) {
                throw new Error("Failed to create withdrawing instruction");
            }
            tx.add(withdrawIx.ix);
            return tx;
        });
    }
    withdrawAfterDelay(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const withdrawPosition = this.pda.withdrawPosition(this.ctx.wallet.publicKey, new anchor_1.BN(index));
            const withdrawPositionInfo = yield this.ctx.connection.getAccountInfo(withdrawPosition.key);
            if (!withdrawPositionInfo) {
                throw new Error("Ensure create withdraw position before getting delay amount");
            }
            const withdrawDelayIx = yield this.ctx.methods.withdrawDelay({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    index,
                },
            });
            if (!withdrawDelayIx.ix) {
                throw new Error("Failed to create withdrawing after delay instruction");
            }
            tx.add(withdrawDelayIx.ix);
            return tx;
        });
    }
    claim() {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const flexibleAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleAccount.key);
            if (!flexibleAccountInfo) {
                throw new Error("Ensure user has staked before claiming");
            }
            const claimIx = yield this.ctx.methods.claim({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
            });
            if (!claimIx.ix) {
                throw new Error("Failed to create claiming instruction");
            }
            tx.add(claimIx.ix);
            return tx;
        });
    }
    lockup(amount, vaultIndex) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const userAccount = this.pda.user(this.ctx.wallet.publicKey);
            const userAccountInfo = yield this.ctx.connection.getAccountInfo(userAccount.key);
            let lockupIndex = new anchor_1.BN(0);
            if (!userAccountInfo) {
                const initUserIx = yield this.ctx.methods.initUser({
                    accounts: {
                        user: this.ctx.wallet.publicKey,
                    },
                    inputs: {
                        bump: userAccount.bump,
                    },
                });
                if (!initUserIx.ix) {
                    throw new Error("Failed to create initializing user instruction");
                }
                tx.add(initUserIx.ix);
            }
            else {
                const userData = yield this.ctx.program.account.user.fetch(userAccount.key);
                lockupIndex = new anchor_1.BN(userData.latestLockupIndex);
            }
            const userLockupStakingAccount = this.pda.lockupStakingAccount(this.ctx.wallet.publicKey, new anchor_1.BN(lockupIndex), new anchor_1.BN(vaultIndex));
            const userLockupStakingAccountInfo = yield this.ctx.connection.getAccountInfo(userLockupStakingAccount.key);
            if (userLockupStakingAccountInfo) {
                throw new Error("Lockup staking account is already initialized");
            }
            const lockUpIx = yield this.ctx.methods.lockup({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    userLockupStakingAccountBump: userLockupStakingAccount.bump,
                    index: lockupIndex.toNumber(),
                    vaultIndex,
                    amount,
                },
            });
            if (!lockUpIx.ix) {
                throw new Error("Failed to create locking up instruction");
            }
            tx.add(lockUpIx.ix);
            return tx;
        });
    }
    unlock(index, vaultIndex) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const unlockIx = yield this.ctx.methods.unlock({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    index,
                    vaultIndex,
                },
            });
            if (!unlockIx.ix) {
                throw new Error("Failed to create unlocking instruction");
            }
            tx.add(unlockIx.ix);
            return tx;
        });
    }
    // Getter functions
    getUserFlexibleStakingInfo(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const flexibleVault = this.pda.flexibleVault();
            const flexibleVaultInfo = yield this.ctx.connection.getAccountInfo(flexibleVault.key);
            if (!flexibleVaultInfo) {
                throw new Error("Flexible vault account not found");
            }
            const flexibleVaultData = yield this.ctx.program.account.flexibleVault.fetch(flexibleVault.key);
            const flexibleStakingAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleStakingAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleStakingAccount.key);
            if (!flexibleStakingAccountInfo) {
                return {
                    totalStakedAmount: "0",
                    currentApr: 0,
                    pendingReward: "0",
                    unstakePending: "0",
                    redeemPending: "0",
                    withdrawPositions: [],
                };
            }
            const flexibleStakingAccountData = yield this.ctx.program.account.flexibleStakingAccount.fetch(flexibleStakingAccount.key);
            const now = Math.floor(Date.now() / 1000);
            const timePassed = new anchor_1.BN(now).sub(flexibleVaultData.lastUpdateTime);
            let rewardPerToken = new anchor_1.BN(0);
            if (flexibleVaultData.totalSupply.eq(new anchor_1.BN(0))) {
                rewardPerToken = flexibleVaultData.rewardPerTokenStored;
            }
            else {
                rewardPerToken = flexibleVaultData.rewardPerTokenStored.add(flexibleVaultData.rewardRate
                    .mul(timePassed)
                    .mul(__1.SCALE_FACTOR)
                    .div(flexibleVaultData.totalSupply));
            }
            const pendingReward = flexibleStakingAccountData.balance
                .mul(rewardPerToken.sub(flexibleStakingAccountData.userRewardPerTokenPaid))
                .div(__1.SCALE_FACTOR)
                .add(flexibleStakingAccountData.reward);
            const filters = [{ dataSize: __1.WITHDRAW_POSITION_LEN }, { memcmp: { offset: (0, __1.getWithdrawPositionUserOffset)(), bytes: this.ctx.wallet.publicKey.toBase58() } }];
            if ((filter === null || filter === void 0 ? void 0 : filter.withdrew) !== undefined) {
                filters.push({
                    memcmp: {
                        offset: (0, __1.getWithdrawPositionClaimedOffset)(),
                        bytes: filter.withdrew ? "2" : "1",
                    },
                });
            }
            const withdrawPositionAccounts = yield this.ctx.connection.getProgramAccounts(__1.STAKING_PROGRAM_ID, { filters });
            const withdrawPositionsInfo = yield Promise.all(withdrawPositionAccounts.map((info) => __awaiter(this, void 0, void 0, function* () {
                const withdrawPositionInfo = yield this.ctx.program.account.withdrawPosition.fetch(info.pubkey);
                return withdrawPositionInfo;
            })));
            let unstakePending = new anchor_1.BN(0);
            let redeemPending = new anchor_1.BN(0);
            const withdrawPositions = withdrawPositionsInfo.map((info) => {
                const releaseTime = info.startDelayTime.add(flexibleVaultData.withdrawalDelayTime);
                if (releaseTime.lt(new anchor_1.BN(now))) {
                    redeemPending = redeemPending.add(info.amount);
                }
                else {
                    unstakePending = unstakePending.add(info.amount);
                }
                return ({
                    index: info.index,
                    amount: info.amount.toString(),
                    startDelayTime: info.startDelayTime.toString(),
                    unlockTime: info.startDelayTime.add(flexibleVaultData.withdrawalDelayTime).toString(),
                    claimed: info.claimed,
                });
            });
            // sort withdraw positions by index
            withdrawPositions.sort((a, b) => a.index - b.index);
            const totalFlexibleStaked = Number(BigInt(flexibleVaultData.totalSupply.toString()));
            const apr = calculateAPR(totalFlexibleStaked, Number(flexibleVaultData.rewardRate.toString()));
            return {
                totalStakedAmount: flexibleStakingAccountData.balance.toString(),
                currentApr: apr,
                pendingReward: pendingReward.toString(),
                unstakePending: unstakePending.toString(),
                redeemPending: redeemPending.toString(),
                withdrawPositions,
            };
        });
    }
    getUserLockupStakingInfo(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const fetcher = new fetcher_1.AccountFetcher(this.ctx.connection);
            const lockupVaultAccounts = yield this.ctx.connection.getProgramAccounts(__1.STAKING_PROGRAM_ID, {
                filters: [
                    {
                        dataSize: __1.LOCKUP_VAULT_LEN
                    }
                ],
            });
            if (lockupVaultAccounts.length === 0) {
                throw new Error("Lockup vault account not found");
            }
            const lockupVaultsInfo = yield Promise.all(lockupVaultAccounts.map((info) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const lockupVaultInfo = fetcher.parseAccountData((_a = info === null || info === void 0 ? void 0 : info.account) === null || _a === void 0 ? void 0 : _a.data, fetcher_1.ParsableLockupVault);
                if (!lockupVaultInfo) {
                    return null;
                }
                return {
                    lockupVault: info.pubkey.toBase58(),
                    bump: lockupVaultInfo.bump,
                    index: lockupVaultInfo.index,
                    apy: lockupVaultInfo.apy,
                    lockupDuration: lockupVaultInfo.lockupDuration.toString(),
                    minStakeAmount: lockupVaultInfo.minStakeAmount.toString(),
                    totalRewardClaimed: lockupVaultInfo.totalRewardClaimed.toString(),
                };
            })));
            lockupVaultsInfo.sort((a, b) => a.index - b.index);
            const filters = [
                { dataSize: __1.LOCKUP_STAKING_ACCOUNT_LEN },
                { memcmp: { offset: (0, __1.getLockupStakingAccountOwnerOffset)(), bytes: this.ctx.wallet.publicKey.toBase58() } },
            ];
            if ((filter === null || filter === void 0 ? void 0 : filter.unlocked) !== undefined) {
                filters.push({
                    memcmp: {
                        offset: (0, __1.getLockupStakingAccountUnlockedOffset)(),
                        bytes: filter.unlocked ? "2" : "1",
                    },
                });
            }
            const lockupStakingAccounts = yield this.ctx.connection.getProgramAccounts(__1.STAKING_PROGRAM_ID, { filters });
            if (lockupStakingAccounts.length === 0) {
                return {
                    totalLockedAmount: "0",
                    maturedAmount: "0",
                    lockupStakingAccounts: [],
                };
            }
            let totalLockedAmount = new anchor_1.BN(0);
            let maturedAmount = new anchor_1.BN(0);
            const lockupStakingAccountsInfo = yield Promise.all(lockupStakingAccounts.map((info) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const lockupStakingAccountInfo = fetcher.parseAccountData((_a = info === null || info === void 0 ? void 0 : info.account) === null || _a === void 0 ? void 0 : _a.data, fetcher_1.ParsableLockupStakingAccount);
                if (!lockupStakingAccountInfo) {
                    return null;
                }
                totalLockedAmount = totalLockedAmount.add(lockupStakingAccountInfo.lockedAmount);
                // find lockup vault info in `lockupVaultsInfo` by lockupStakingAccountInfo.lockupVault
                const lockupVaultInfo = lockupVaultsInfo.find((vault) => vault.lockupVault === lockupStakingAccountInfo.lockupVault.toBase58());
                if (!lockupVaultInfo) {
                    throw new Error("User has an invalid lockup vault");
                }
                else {
                    const maturedTime = lockupStakingAccountInfo.lockedAt.add(new anchor_1.BN(lockupVaultInfo.lockupDuration.toString()));
                    if (maturedTime.lte(new anchor_1.BN(Math.floor(Date.now() / 1000)))) {
                        maturedAmount = maturedAmount.add(lockupStakingAccountInfo.lockedAmount);
                    }
                }
                return {
                    bump: lockupStakingAccountInfo.bump,
                    index: lockupStakingAccountInfo.index,
                    lockupVault: lockupStakingAccountInfo.lockupVault.toBase58(),
                    lockupVaultIndex: lockupVaultInfo.index,
                    committedApy: lockupStakingAccountInfo.committedApy,
                    lockedAmount: lockupStakingAccountInfo.lockedAmount.toString(),
                    lockedAt: lockupStakingAccountInfo.lockedAt.toString(),
                    unlocked: lockupStakingAccountInfo.unlocked,
                };
            })));
            // sort lockup staking accounts info by index
            lockupStakingAccountsInfo.sort((a, b) => a.index - b.index);
            return {
                totalLockedAmount: totalLockedAmount.toString(),
                maturedAmount: maturedAmount.toString(),
                lockupStakingAccounts: lockupStakingAccountsInfo,
            };
        });
    }
}
exports.StakingClient = StakingClient;
function getProtocolInfo(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const pda = new __1.PDA(__1.STAKING_PROGRAM_ID);
        const fetcher = new fetcher_1.AccountFetcher(connection);
        const config = pda.config();
        const configData = yield fetcher.getConfig(config.key, true);
        if (!configData) {
            throw new Error("Config account not found");
        }
        const flexibleVault = pda.flexibleVault();
        const flexibleVaultInfo = yield fetcher.getFlexibleVault(flexibleVault.key, true);
        if (!flexibleVaultInfo) {
            throw new Error("Flexible vault account not found");
        }
        const lockupVaultAccounts = yield connection.getProgramAccounts(__1.STAKING_PROGRAM_ID, {
            filters: [
                {
                    dataSize: __1.LOCKUP_VAULT_LEN
                }
            ],
        });
        if (lockupVaultAccounts.length === 0) {
            throw new Error("Lockup vault account not found");
        }
        const lockupVaultsInfo = yield Promise.all(lockupVaultAccounts.map((info) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const lockupVaultInfo = fetcher.parseAccountData((_a = info === null || info === void 0 ? void 0 : info.account) === null || _a === void 0 ? void 0 : _a.data, fetcher_1.ParsableLockupVault);
            return lockupVaultInfo;
        })));
        // sort lockup vaults info by index
        lockupVaultsInfo.sort((a, b) => a.index - b.index);
        const totalFlexibleStaked = Number(BigInt(flexibleVaultInfo.totalSupply.toString()));
        const apr = calculateAPR(totalFlexibleStaked, Number(flexibleVaultInfo.rewardRate.toString()));
        return Object.assign({ admin: configData.admin.toBase58(), mint: configData.mint.toBase58(), flexibleApr: apr, flexibleWithdrawalDelayTime: flexibleVaultInfo.withdrawalDelayTime.toString(), totalFlexibleStaked: totalFlexibleStaked.toString(), totalFlexibleReward: flexibleVaultInfo.totalRewardClaimed.toString() }, lockupVaultsInfo.reduce((acc, vault) => {
            if (vault) {
                acc.lockupApy.push(Number(vault.apy));
                acc.lockupDuration.push(vault.lockupDuration.toString());
                acc.minLockupAmount.push(vault.minStakeAmount.toString());
                acc.totalLockupStaked.push(vault.totalLockedAmount.toString());
                acc.totalLockupReward.push(vault.totalRewardClaimed.toString());
            }
            return acc;
        }, {
            lockupApy: [],
            lockupDuration: [],
            minLockupAmount: [],
            totalLockupStaked: [],
            totalLockupReward: [],
        }));
    });
}
function calculateAPR(totalStaked, rewardPerSecond) {
    if (totalStaked === 0)
        return 0;
    const secondsInYear = 365 * 24 * 60 * 60;
    const annualReward = rewardPerSecond * secondsInYear;
    const apr = (annualReward / totalStaked) * 100;
    return apr;
}
