import { TransactionInstruction } from "@solana/web3.js";
import { Context } from "./context";
import * as ixs from "./instructions";

export class Methods {
  public ctx: Context;
  public ix: TransactionInstruction | null | undefined;

  public constructor(ctx: Context, ix?: TransactionInstruction) {
    this.ctx = ctx;
    this.ix = ix;
  }

  // Admin functions
  public async updateAdmin(params: ixs.UpdateAdminParams) {
    this.ix = await ixs.updateAdmin(this.ctx.program, params);
    return this;
  }

  public async withdrawFund(params: ixs.WithdrawFundParams) {
    this.ix = await ixs.withdrawFund(this.ctx.program, params);
    return this;
  }

  public async updateLockupVault(params: ixs.UpdateLockupVaultParams) {
    this.ix = await ixs.updateLockupVault(this.ctx.program, params);
    return this;
  }

  public async updateFlexibleVault(params: ixs.UpdateFlexibleVaultParams) {
    this.ix = await ixs.updateFlexibleVault(this.ctx.program, params);
    return this;
  }

  // User functions
  public async initUser(params: ixs.InitUserParams) {
    this.ix = await ixs.initUser(this.ctx.program, params);
    return this;
  }

  public async initFlexibleAccount(params: ixs.InitFlexibleAccountParams) {
    this.ix = await ixs.initFlexibleAccount(this.ctx.program, params);
    return this;
  }

  public async stake(params: ixs.StakeParams) {
    this.ix = await ixs.stake(this.ctx.program, params);
    return this;
  }

  public async withdraw(params: ixs.WithdrawParams) {
    this.ix = await ixs.withdraw(this.ctx.program, params);
    return this;
  }

  public async withdrawDelay(params: ixs.WithdrawDelayParams) {
    this.ix = await ixs.withdrawDelay(this.ctx.program, params);
    return this;
  }

  public async claim(params: ixs.ClaimParams) {
    this.ix = await ixs.claim(this.ctx.program, params);
    return this;
  }

  public async lockup(params: ixs.LockupParams) {
    this.ix = await ixs.lockup(this.ctx.program, params);
    return this;
  }

  public async unlock(params: ixs.UnlockParams) {
    this.ix = await ixs.unlock(this.ctx.program, params);
    return this;
  }
}
